/* Deep Waters v0.9.0 */
define('deep-waters',['exports'],function(exports){'use strict';/**
 * Takes an object of validators defining the possible shape of an object and returns a new function that reports
 * whether the received objects matches that shape or not.
 *
 * Please note:
 * The created validator isUser only makes sure the received object has all the defined keys and each key matches
 * the defined validation. It does not check on properties not defined by the shape, meaning it will return true
 * even if the object has additional properties.
 *
 * @param shape
 * @returns {function(*): boolean}
 */
var hasShape = function hasShape(shape) {
  return function (object) {
    return Object.keys(shape).every(function (prop) {
      var validator = shape[prop];
      var result = false;

      try {
        result = validator(object[prop]);
      } catch (e) {
        return false;
      }

      return result;
    });
  };
};/**
 * It's the very backbone of Deep Waters, it takes a series of validators and returns a new function that takes a
 * value and evaluates whether that value is valid for all the previously provided validators.
 * @param validators
 * @returns {function(*=): boolean}
 */
var compose = function compose() {
  for (var _len = arguments.length, validators = new Array(_len), _key = 0; _key < _len; _key++) {
    validators[_key] = arguments[_key];
  }

  return function (value) {
    return Array.from(new Set(validators)).every(function (validator) {
      return validator(value);
    });
  };
};/**
 * Using Object.prototype.toString.call() works much than typeof for type checking in JavaScript, since it works
 * properly for null. Not only that, but it returns the specific type of any given object.
 * @param className
 * @returns {function(*=): boolean}
 */
var ofClass = function ofClass(className) {
  return function (value) {
    return Object.prototype.toString.call(value) === "[object ".concat(className, "]");
  };
};/**
 * Returns true if the given value is of type function
 *
 * According to: https://stackoverflow.com/questions/5999998/check-if-a-variable-is-of-function-type
 * @param fn
 * @returns {boolean|boolean}
 */

var isFunction = ofClass('Function');/**
 * Makes sure that every item of the array is valid for the received validator
 * @param validator
 * @returns {function(*): *}
 */

var every = function every(validator) {
  return compose(function () {
    return isFunction(validator);
  }, Array.isArray, function (items) {
    return items.every(validator);
  });
};/**
 * Takes an object of validators defining the possible shape of an object and returns a new function that reports
 * whether the received array matches that shape for every of its elements.
 * @param shape
 * @returns {function(*): *}
 */

var arrayOfShape = function arrayOfShape(shape) {
  return every(hasShape(shape));
};/**
 * Takes a value and return true if it's of type number
 * @param value
 * @returns {boolean}
 */

var isNumber = ofClass('Number');/**
 * Evaluates whether the received value is a number bigger than the defined one
 * @param number
 * @returns {function(*=): boolean}
 */

var biggerThan = function biggerThan(number) {
  return compose(isNumber, function (value) {
    return value > number;
  });
};/**
 * Takes a number N and return a new function that evaluates whether the received value is of type number and it's
 * smaller than N (the one previously defined). The evaluation is not inclusive.
 * @param number
 * @returns {function(*=): boolean}
 */

var smallerThan = function smallerThan(number) {
  return compose(isNumber, function (value) {
    return value < number;
  });
};/**
 * Evaluates whether the received value is between at defined range
 * @param min
 * @returns {function(*=): function(*=): boolean}
 */

var between = function between(min) {
  return function (max) {
    return compose(biggerThan(min), smallerThan(max));
  };
};/**
 * Takes a series of validator functions and return a new functions that evaluate whether the given value
 * is a valid value for at least one of the previous validators.
 * @param validators
 * @returns {function(*=): boolean}
 */
var or = function or() {
  for (var _len = arguments.length, validators = new Array(_len), _key = 0; _key < _len; _key++) {
    validators[_key] = arguments[_key];
  }

  return function (value) {
    return validators.reduce(function (acc, current) {
      return acc || current(value);
    }, false);
  };
};var when = function when(condition, fn) {
  return function (value) {
    if (condition(value)) return fn(value);
    return true;
  };
};/**
 * Takes a function and return a new function which will return the opposite of the given one
 * @param fn
 * @returns {function(*=): boolean}
 */
var not = function not(fn) {
  return function (value) {
    return !fn(value);
  };
};/**
 * Evaluates whether the given value is of type string
 * @param value
 * @returns {boolean}
 */

var isString = ofClass('String');/**
 * Evaluates whether the given value is of type bigint
 * @param value
 * @returns {boolean}
 */

var isBigInt = ofClass('BigInt');/**
 * returns true if the given value is a valid boolean
 * @param value
 * @returns {boolean}
 */

var isBoolean = ofClass('Boolean');/**
 * Evaluates whether the given value is of type symbol
 * @param value
 * @returns {boolean}
 */

var isSymbol = ofClass('Symbol');/**
 * Takes a value and returns true if it's undefined
 * @param value
 * @returns {boolean}
 */

var isUndefined = ofClass('Undefined');/**
 * Checks if a given variable is null
 * @param value
 * @returns {boolean}
 */

var isNull = ofClass('Null');/**
 * Receives a value and reports whether it is a primitive of not
 * @type {function(*=): boolean}
 */

var isPrimitive = or(isString, isFunction, isBoolean, isNumber, isBigInt, isSymbol, isNull, isUndefined);/**
 * Returns true if (and only) the given value is a valid object
 * @type {function(*=): boolean}
 */

var isObject = ofClass('Object');var isDate = ofClass('Date');var objectEquals = function objectEquals(object) {
  return function (value) {
    if (!isObject(object)) return false;
    var objKeys = Object.keys(object);
    var valueKeys = Object.keys(value);
    if (objKeys.length !== valueKeys.length) return false; // eslint-disable-next-line no-use-before-define

    return valueKeys.every(function (key) {
      return equals(value[key])(object[key]);
    });
  };
}; // eslint-disable-next-line no-use-before-define


var arrayEquals = function arrayEquals(array) {
  return every(function (item, index) {
    return equals(item)(array[index]);
  });
};
/**
 * Takes a value val, of any type, and returns a new function evaluating that the received value equals val
 * (the one previously defined).
 * @param value
 * @returns {function(*=): boolean}
 */


var equals = function equals(value) {
  return compose(when(or(isPrimitive), function (string) {
    return value === string;
  }), when(isDate, function () {
    return !!value.valueOf();
  }), when(Array.isArray, arrayEquals(value)), when(isObject, objectEquals(value)));
};/**
 * Honestly, I'm not going to comment this function
 * @returns {boolean}
 */
var stubFalse = function stubFalse() {
  return false;
};var arrayContains = function arrayContains(value) {
  return function (array) {
    if (isPrimitive(value)) {
      return array.includes(value);
    }

    return array.filter(not(equals(value))).length !== array.length;
  };
};
/**
 * Evaluates whether the specified value equals at least one of the elements of the given list, if the provided
 * list is otherwise a string, makes sure that value it's a sub-string
 */


var contains = function contains(value) {
  return compose(when(isString, function (string) {
    return string.includes(value);
  }), when(Array.isArray, arrayContains(value)), when(not(or(isString, Array.isArray)), stubFalse));
};/**
 * Takes a value and returns true only if it is of type boolean and of value `false`
 * @param value
 * @returns {boolean}
 */
var isFalse = function isFalse(value) {
  return value === false;
};/**
 * Returns true if the given value is zero
 * @param value
 * @returns {boolean}
 */
var isZero = function isZero(value) {
  return value === 0;
};/**
 * Takes a value and returns true only if it's of type `string` and it's empty
 * @type {function(*=): boolean}
 */

var isEmptyString = compose(isString, function (str) {
  return str.trim() === '';
});/**
 * Returns true if the given value is falsy (false, zero, empty string, null, undefined or NaN)
 * @type {function(*=): *|boolean}
 */

var isFalsy = or(isFalse, isZero, isEmptyString, isNull, isUndefined, function (value) {
  return !value;
});/**
 * Takes a value and returns true if it's not falsy
 * @param value
 * @returns {boolean}
 */

var isTruthy = not(isFalsy);/**
 * Takes a string defining a property name and returns a new function evaluation whether the received object exists and
 * has the defined property.
 * @param prop
 * @returns {function(*=): boolean}
 */

var hasProp = function hasProp(prop) {
  return compose(isTruthy, function (value) {
    return prop && value[prop] !== undefined;
  });
};var isUint8Array = ofClass('Uint8Array');var isBuffer = isUint8Array;var isDivisibleBy = function isDivisibleBy(number) {
  return compose(isNumber, function (value) {
    return value % number === 0;
  });
};var matchesPattern = function matchesPattern(regex) {
  return function (str) {
    return !!regex.test(str);
  };
};var isEmail = compose(isString, matchesPattern(/^\w+([.-]?\w+)*@\w+([.-]?\w+)*(\.\w{2,3})+$/));/**
 * Takes number and returns a function that returns true if the length property of its first parameter
 * (may it be an array or a string) equals the given number.
 * @param number
 * @returns {function({length: *}): boolean}
 */

var ofLength = function ofLength(number) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length === number;
  });
};/**
 * Returns true if the given value is empty (can be a string, an array or an object)
 * @type {function(*=): boolean}
 */

var isEmpty = compose(when(Array.isArray, ofLength(0)), when(isString, isEmptyString), when(isObject, function (value) {
  return isEmpty(Object.keys(value));
}));var isError = ofClass('Error');var isInt8Array = ofClass('Int8Array');var ipv4Regex = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
/**
 * Takes a value and return true if it's a valid IP v4 string
 * @param value
 * @returns {boolean}
 */

var isIPv4 = compose(isString, matchesPattern(ipv4Regex));var ipv6Regex = /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/;
/**
 * Takes a value and return true if it's a valid IP v6 string
 * @param value
 * @returns {boolean}
 */

var isIPv6 = compose(isString, matchesPattern(ipv6Regex));/**
 * Takes a value and return true if it's a valid IP (v4 or v6) string
 * @param value
 * @returns {boolean}
 */

var isIP = or(isIPv4, isIPv6);var isMap = ofClass('Map');var isNumeric = compose(isString, matchesPattern(/^-{0,1}\d+$/));/* eslint-disable max-len */

var isInternational = matchesPattern(/\+(9[976]\d|8[987530]\d|6[987]\d|5[90]\d|42\d|3[875]\d|2[98654321]\d|9[8543210]|8[6421]|6[6543210]|5[87654321]|4[987654310]|3[9643210]|2[70]|7|1)\d{1,14}$/);
var isUK = matchesPattern(/^(?:(?:\(?(?:0(?:0|11)\)?[\s-]?\(?|\+)44\)?[\s-]?(?:\(?0\)?[\s-]?)?)|(?:\(?0))(?:(?:\d{5}\)?[\s-]?\d{4,5})|(?:\d{4}\)?[\s-]?(?:\d{5}|\d{3}[\s-]?\d{3}))|(?:\d{3}\)?[\s-]?\d{3}[\s-]?\d{3,4})|(?:\d{2}\)?[\s-]?\d{4}[\s-]?\d{4}))(?:[\s-]?(?:x|ext\.?|#)\d{3,4})?$/);
var isUSA = matchesPattern(/1?\W*([2-9][0-8][0-9])\W*([2-9][0-9]{2})\W*([0-9]{4})(\se?x?t?(\d*))?/);
var isGermany = matchesPattern(/^([+][0-9]{1,3}[ .-])?([(]{1}[0-9]{1,6}[)])?([0-9 .\-/]{3,20})((x|ext|extension)[ ]?[0-9]{1,4})?$/);
var isChina = matchesPattern(/1[34578][012356789]\d{8}|134[012345678]\d{7}/);
var isRussia = matchesPattern(/^((\+7|7|8)+([0-9]){10})$/);
var isJapan = matchesPattern(/^(?:\d{10}|\d{3}-\d{3}-\d{4}|\d{2}-\d{4}-\d{4}|\d{3}-\d{4}-\d{4})$/);
/* eslint-enable max-len */

var isPhone = compose(isString, or(isInternational, isUSA, isUK, isGermany, isUSA, isChina, isRussia, isJapan));var isRegExp = ofClass('RegExp');/**
 * Takes a value and returns true if it exists;
 * It's a semantic variation of `isTruthy`
 * @param value
 * @returns {boolean}
 */

var isRequired = isTruthy;var isSet = ofClass('Set');/**
 * Takes a value and returns true only if it is of type boolean and of value `true`
 * @param value
 * @returns {boolean}
 */
var isTrue = function isTrue(value) {
  return value === true;
};/**
 * Returns true if the given string matches a valid url, according to:
 * https://stackoverflow.com/questions/5717093/check-if-a-javascript-string-is-a-url
 * @type {function(*=): boolean}
 */

var isURL = compose(not(isEmptyString), matchesPattern(new RegExp('^(https?:\\/\\/)?' // protocol
+ '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' // domain name
+ '((\\d{1,3}\\.){3}\\d{1,3}))' // OR ip (v4) address
+ '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' // port and path
+ '(\\?[;&a-z\\d%_.~+=-]*)?' // query string
+ '(\\#[-a-z\\d_]*)?$', 'i')));var isWeakMap = ofClass('WeakMap');var isWeakSet = ofClass('WeakSet');var maxLength = function maxLength(max) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length <= max;
  });
};var minLength = function minLength(min) {
  return compose(hasProp('length'), function (_ref) {
    var length = _ref.length;
    return length >= min;
  });
};var ofUniqueItems = compose(Array.isArray, function (array) {
  return new Set(array).size === array.length;
});// TODO: extend to support objects and other arrays
var oneOfValues = function oneOfValues(array) {
  return function (value) {
    return Array.isArray(array) ? array.includes(value) : false;
  };
};exports.arrayOfShape=arrayOfShape;exports.between=between;exports.biggerThan=biggerThan;exports.compose=compose;exports.contains=contains;exports.equals=equals;exports.every=every;exports.hasProp=hasProp;exports.hasShape=hasShape;exports.isBigInt=isBigInt;exports.isBoolean=isBoolean;exports.isBuffer=isBuffer;exports.isDate=isDate;exports.isDivisibleBy=isDivisibleBy;exports.isEmail=isEmail;exports.isEmpty=isEmpty;exports.isEmptyString=isEmptyString;exports.isError=isError;exports.isFalse=isFalse;exports.isFalsy=isFalsy;exports.isFunction=isFunction;exports.isIP=isIP;exports.isIPv4=isIPv4;exports.isIPv6=isIPv6;exports.isInt8Array=isInt8Array;exports.isMap=isMap;exports.isNull=isNull;exports.isNumber=isNumber;exports.isNumeric=isNumeric;exports.isObject=isObject;exports.isPhone=isPhone;exports.isPrimitive=isPrimitive;exports.isRegExp=isRegExp;exports.isRequired=isRequired;exports.isSet=isSet;exports.isString=isString;exports.isSymbol=isSymbol;exports.isTrue=isTrue;exports.isTruthy=isTruthy;exports.isURL=isURL;exports.isUint8Array=isUint8Array;exports.isUndefined=isUndefined;exports.isWeakMap=isWeakMap;exports.isWeakSet=isWeakSet;exports.isZero=isZero;exports.matchesPattern=matchesPattern;exports.maxLength=maxLength;exports.minLength=minLength;exports.not=not;exports.ofClass=ofClass;exports.ofLength=ofLength;exports.ofUniqueItems=ofUniqueItems;exports.oneOfValues=oneOfValues;exports.or=or;exports.smallerThan=smallerThan;exports.when=when;Object.defineProperty(exports,'__esModule',{value:true});});//# sourceMappingURL=deep-waters.amd.js.map
